import matplotlib.pyplot as plt
from numpy import zeros, pi, log, linspace, meshgrid

def getDZfromHeatFlux(Q,d,D,h1,h2,dT,k):
    """
    Výpočet výšky elementu ze známého výkonu a tepelného spádu
    ---
    Q - tepelný výkon elementu [W]
    d - vnitřní průměr stěny [m]
    D - vnější průměr stěny [m]
    h1 - součinitel přestupu tepla na primární straně [W/m2/K]
    h2 - součinitel přestupu tepla na sekundární straně [W/m2/K]
    dT - tepelný spád [°C]
    k - tepelná vodivost stěny [W/m/K]
    """
    U = U_cylinder(d,D,h1,h2,k)
    dz = Q / (dT*U)
    return dz

def U_cylinder(d,D,h1,h2,k):
    """
    Součinitel prostupu tepla pro válcovou geometrii
    d - vnitřní průměr [m]
    D - vnější průměr [m]
    h1 - součinitel přestupu tepla na vnitří straně [W/m2/K]
    h2 - součinitel přestupu tepla na vnější straně [W/m2/K]
    -->
    U - součinitel prostupu tepla [W/m/K]
    """
    U = pi / (1/(h1*d) + 1/(2*k)*log(D/d) + 1/(h2*D))
    return U

class GeometryClass:
    """
    Třída obsahující, co se týká geometrie pasti a jejího materiálu
    """
    def __init__(self, D1, D2, D3, D4, D5, D6, D7, tl, k, T_cool, P_cool):
        # Průměry [m]
        self.D_in_I = D1
        self.D_out_I = D2
        self.D_in_II = D3
        self.D_sep = D4
        self.D_out_II = D5
        
        # tercial
        self.D_cool = D6
        self.d_cool = D7
        self.tl_cool = tl
        self.T_cool = T_cool
        self.P_cool = P_cool*1e5
        
        # Tepelná vodivost [W/m/K]
        self.k = k
        
        # Dopočtené hodnoty ------
        self.dz_sep = (self.D_sep - self.D_in_II)/2
        
        # průtočné průřezy
        self.A_I = pi*(self.D_out_I**2 - self.D_in_I**2)/4
        self.A_II_in = pi*(self.D_sep**2 - self.D_in_II**2)/4
        self.A_II_out = pi*(self.D_out_II**2 - self.D_sep**2)/4
        
        # hydraulické průměry
        self.Dh_I = self.D_out_I - self.D_in_I
        self.Dh_II_in = self.D_sep - self.D_in_II
        self.Dh_II_out = self.D_out_II - self.D_sep
    
    def solveHeight(self, P_i, h_eu_i, h_water_i, dT_eu_water,x):
        # Výpočet výšky mezi uzly
        self.dZ = getDZfromHeatFlux(P_i, self.D_out_I, self.D_in_II, h_eu_i, h_water_i, dT_eu_water, self.k)
        n = len(h_eu_i)
        self.Z = zeros(n+1)
        self.z_NB = None
        self.i_NB = None
        for i in range(1,n+1):
            self.Z[i] = self.Z[i-1]+self.dZ[i-1]
            if x[i] > 0 and x[i-1] == 0:
                self.z_NB = (self.Z[i]+self.Z[i-1])/2
                self.i_NB = i-0.5
        
        # průtočné plochy pro teplo mezi výpočtovými uzly
        self.A_water_internodal = pi * self.D_in_II * self.dZ
        self.A_eu_internodal = pi * self.D_out_I * self.dZ
    
    # ~ def drawGeometry(self, bubbleRadius, alpha, T_wall):
        # ~ # Schéma pasti --------------------------------------------
        # ~ figScheme = plt.figure("Scheme")
        # ~ plt.title("Schéma studené pasti")
        
        # ~ try:
            # ~ nr = T_wall.shape[1]
            # ~ R = linspace(self.D_out_I/2, self.D_in_II/2, num=nr)
            # ~ X,Y = meshgrid(R,self.Z)

            # ~ plt.contourf(X,Y,T_wall,50,cmap=plt.get_cmap("hot"))
            #plt.colorbar()
            # ~ #plt.gca().set_aspect('equal', adjustable='box')
            #plt.axis('scaled')
            # ~ cb = plt.colorbar()
            # ~ cb.set_label('Teplota stěny [°C]')
        # ~ except:
            # ~ plt.fill_between((self.D_out_I/2,self.D_in_II/2),(0,0),(self.Z[-1],self.Z[-1]), color="black", alpha=.2, label="ocel")
        
        # ~ plt.fill_between((self.D_in_II/2,self.D_out_II/2),(0,0),(self.Z[-1],self.Z[-1]), color="blue", label="H2O", alpha=0.7)
        # ~ plt.fill_between((self.D_in_I/2,self.D_out_I/2),(0,0),(self.Z[-1],self.Z[-1]), color="red", label="LiPb", alpha=0.7)

        #plt.fill_between((self.D_out_I/2,self.D_in_II/2),(0,0),(self.Z[-1],self.Z[-1]), color="black", alpha=.2, label="ocel")
        
        # ~ skip_i = int((bubbleRadius*2)/self.dZ[-1])
        
        # ~ import random
        # ~ for i in range(len(alpha))[::skip_i]:
            # ~ n_bubbles_float = ((self.D_sep-self.D_in_II)/2)/(bubbleRadius*2)*alpha[i]
            # ~ n_bubbles = int(round(n_bubbles_float,0))
            # ~ if n_bubbles == 0 and alpha[i] != 0:
                # ~ n_bubbles = 1
            # ~ z = self.Z[i]
            # ~ for b in range(n_bubbles):
                # ~ r = random.uniform(self.D_in_II/2+bubbleRadius,self.D_sep/2-bubbleRadius)
                # ~ circle=plt.Circle((r,z),bubbleRadius,color='w',alpha=.8)
                # ~ plt.gcf().gca().add_artist(circle)

        # ~ plt.plot((0,self.D_in_I/2,self.D_in_I/2,0),(0,0,self.Z[-1],self.Z[-1]), color="black")
        # ~ plt.plot((self.D_out_II/2+0.01,self.D_out_II/2,self.D_out_II/2,self.D_out_II/2+0.01),(0,0,self.Z[-1],self.Z[-1]), color="black")
        # ~ plt.plot((self.D_out_I/2, self.D_in_II/2, self.D_in_II/2, self.D_out_I/2, self.D_out_I/2),(0,0,self.Z[-1],self.Z[-1],0), color="black")
        # ~ plt.plot((self.D_in_II/2, self.D_out_II/2),(0, 0), color="black")

        # ~ plt.plot((0,0),(-0.01,self.Z[-1]+0.01), "-.", color="black")

        # ~ plt.plot((self.D_sep/2, self.D_sep/2),(self.dz_sep, self.Z[-1]-self.dz_sep), color="black")

        # ~ plt.plot((self.D_in_II/2, self.D_out_II/2), (self.z_NB, self.z_NB), label="z NB", color="red", ls="--")
        
        # ~ plt.xlabel("Poloměr [m]")
        # ~ plt.ylabel("Výška [m]")
        # ~ plt.axis('equal')
        #plt.axis('scaled')
        # ~ plt.xlim(0, self.D_out_II/2+0.01)
        # ~ plt.legend()
    def drawGeometry(self, bubbleRadius, alpha, T_wall, plotObject, masterplot, wall):
        # Schéma pasti --------------------------------------------
        plotObject.set_title("Schéma studené pasti")
        
        if wall:
            nr = T_wall.shape[1]
            R = linspace(self.D_out_I/2, self.D_in_II/2, num=nr)
            X,Y = meshgrid(R,self.Z)

            cont = plotObject.contourf(X,Y,T_wall,50,cmap=plt.get_cmap("hot"))
            cb = masterplot.colorbar(cont)
            cb.set_label('Teplota stěny [°C]')
        else:
            plotObject.fill_between((self.D_out_I/2,self.D_in_II/2),(0,0),(self.Z[-1],self.Z[-1]), color="black", alpha=.2, label="ocel")
        
        plotObject.fill_between((self.D_in_II/2,self.D_out_II/2),(0,0),(self.Z[-1],self.Z[-1]), color="blue", label="H2O", alpha=0.7)
        plotObject.fill_between((self.D_in_I/2,self.D_out_I/2),(0,0),(self.Z[-1],self.Z[-1]), color="red", label="LiPb", alpha=0.7)
        
        skip_i = int((bubbleRadius*2)/self.dZ[-1])
        
        import random
        for i in range(len(alpha))[::skip_i]:
            n_bubbles_float = ((self.D_sep-self.D_in_II)/2)/(bubbleRadius*2)*alpha[i]
            n_bubbles = int(round(n_bubbles_float,0))
            if n_bubbles == 0 and alpha[i] != 0:
                n_bubbles = 1
            z = self.Z[i]
            for b in range(n_bubbles):
                r = random.uniform(self.D_in_II/2+bubbleRadius,self.D_sep/2-bubbleRadius)
                circle=plt.Circle((r,z),bubbleRadius,color='w',alpha=.8)
                plotObject.add_artist(circle)
        
        # ~ circle=plt.Circle((self.D_cool/2,self.Z[-1]+self.d_cool/2+0.005),self.d_cool/2,color='black')
        # ~ plotObject.add_artist(circle)
        # ~ circle=plt.Circle((self.D_cool/2,self.Z[-1]+self.d_cool/2+0.005),(self.d_cool-2*self.tl_cool)/2,color='blue')
        # ~ plotObject.add_artist(circle)

        plotObject.plot((0,self.D_in_I/2,self.D_in_I/2,0),(0,0,self.Z[-1],self.Z[-1]), color="black")
        plotObject.plot((self.D_out_II/2+0.01,self.D_out_II/2,self.D_out_II/2,self.D_out_II/2+0.01),(0,0,self.Z[-1],self.Z[-1]), color="black")
        plotObject.plot((self.D_out_I/2, self.D_in_II/2, self.D_in_II/2, self.D_out_I/2, self.D_out_I/2),(0,0,self.Z[-1],self.Z[-1],0), color="black")
        plotObject.plot((self.D_in_II/2, self.D_out_II/2),(0, 0), color="black")

        plotObject.plot((0,0),(-0.01,self.Z[-1]+0.01), "-.", color="black")

        plotObject.plot((self.D_sep/2, self.D_sep/2),(self.dz_sep, self.Z[-1]-self.dz_sep), color="black")

        plotObject.plot((self.D_in_II/2, self.D_out_II/2), (self.z_NB, self.z_NB), label="z NB", color="red", ls="--")
        
        plotObject.set_xlabel("Poloměr [m]")
        plotObject.set_ylabel("Výška [m]")
        plotObject.axis('equal')
        plotObject.set_xlim(0, self.D_out_II/2+0.01)
        plotObject.legend()
