from numpy import linspace, zeros
import SinglePhaseFlow

def Density(T):
    """
    Hustota eutektika [kg/m3]
    T [°C]
    """
    rho = (10.45*(1-161e-6*(T+273.15)))*1e3
    # ~ rho = 9499-.85*(T-235)
    # ~ rho = 9482-1.145*(T-235)
    # ~ rho = 9495-.695*(T-235)
    # ~ rho = 9519-.71*(T-235)
    
    return rho

def Capacity(T):
    """
    Tepelná kapacita eutektika [J/kg/K]
    T [°C]
    """ 
    
    # ~ cp = 192.45-8.9487e-3*T
    # ~ cp = 195-9.116e-6*(T+273.15)
    cp = 190.4-.925e-2*(T-235)
    
    return cp

def Conductivity(T):
    """
    Tepelná vodivost [W/m/K]
    T [°C]
    """
    
    k = (1.95e-2+19.6e-5*(T+273.15))*1e2
    # ~ k = 7.3008+1.96e-2*T
    # ~ k = 11.9+1.96e-2*(T-235)
    
    return k

def Viscosity(T):
    """
    Dynamická viskozita [Pa.s]
    T [°C]
    """
    
    # ~ mu = (0.187*np.exp(11640/(8.314*(T+273.15))))/1e3
    e = 2.718281828459045
    mu = (0.187*e**(11640/(8.314*(T+273.15))))/1e3
    
    # ~ mu = 0.1828*np.exp(11400/(8.314*(T+273.15)))/1e3
    
    return mu

def Enthalpy(x, var):
    """
    entalpie eutektika
    ---
    var - volba výstupní proměnné:
    "H" = entalpie - potom x = teplota [°C] --> vrací entalpii [J/kg]
    "T" = teplota - potom x = entalpie [J/kg] --> vrací teplotu [°C]
    """
    T_m = 235
    
    if var == "H":
        x -= 235
    
    A = 190.4
    B = -0.925e-2
    
    if var == "H":
        H = A*x + B/2*x**2
        return H
    elif var == "T":
        D = A**2 + 2*B*x
        T1 = (-A+D**.5)/B
        T1 += 235
        return T1

def HeatPower(Tin, Tout, m):
    """
    Tepelný výkon [W]
    Tin, Tout - vstupní, výstupní teplota eutektika [°C]
    m - průtok eutektika [kg/s]
    """
    h_in = Enthalpy(Tin,"H")
    h_out = Enthalpy(Tout,"H")
    dh = abs(h_out-h_in)
    P = m*dh
    return dh, P

def hEutecticum(euClass, dz, Dh):
    """
    euClassIn/euClassIn třídy eutektika na vstupu/výstupu
    dz - výška úseku [m]
    Dh - hydraulický průměr [m]
    --->
    h - součinitel přestupu tepla [W/m2/K]
    """
    epsilon = 1.72*(Dh/dz)**.16
    Re = euClass.Re
    Pr = euClass.Pr
    k_mean = euClass.k
    if 20 < Re*Pr < 300 and dz/Dh<30:
        Nu = epsilon*.7*(Re*Pr)**.33
        h = Nu*k_mean/Dh
        return h
    elif not 20 < Re*Pr < 300 and dz/Dh<30:
        print("h-Eutektikum: Re*Pr mimo oblast platnosti korelace")
        print("Pr*Re = ", Re*Pr)
        return None
    elif 20 < Re*Pr < 300 and dz/Dh>30:
        print("h-Eutektikum: L/D mimo oblast platnosti korelace")
        return None
    else:
        print("h-Eutektikum: Parametry mimo oblast platnosti korelace")
        return None

# Třídy -----------------------------------------------
class EutecticumClass:
    def __init__(self, T):
        self.T = T
        self.rho = Density(T)
        self.cp = Capacity(T)
        self.k = Conductivity(T)
        self.mu = Viscosity(T)
        self.Pr = self.cp*self.mu/self.k
        self.h = Enthalpy(T,"H")
    
    def velocity(self, A, m):
        self.m = m
        self.c = m/(A*self.rho)
    
    def Reynolds(self, Dh):
        self.Re = self.rho*Dh*self.c/self.mu

class HotSideClass:
    def __init__(self, T_in, T_out, m, n_node, A, Dh):
        self.T_in = T_in
        self.T_out = T_out
        self.m = m
        
        self.dH_total, self.Power = HeatPower(T_in, T_out, m)
        
        self.H = linspace(EutecticumClass(T_out).h, EutecticumClass(T_in).h, num=n_node+1)
        self.T = zeros(n_node+1)
        self.Eu = []

        for i in range(n_node+1):
            self.T[i] = Enthalpy(self.H[i], "T")
            
            self.Eu.append(EutecticumClass(self.T[i]))
            self.Eu[i].velocity(A,self.m)
            self.Eu[i].Reynolds(Dh)
        
        self.dH = abs(self.H[0] - self.H[1])
        self.P_i = self.dH * self.m
        
        self.T_internodal = (self.T[0:-1] + self.T[1:])/2
    
    def heatTransfer(self, q, Dh):
        n = len(self.T)
        self.h = zeros(n)
        self.dT_w = zeros(n)
        for i in range(n):
            self.h[i], self.dT_w[i] = SinglePhaseFlow.heatTrasferRate(self.Eu[i].Re, self.Eu[i].Pr, self.Eu[i].k, Dh, q[i])
        
        self.h_internodal = (self.h[0:-1] + self.h[1:])/2
    
    def solveHeatFlux(self, A_i):
        n = len(self.H)
        self.q_internodal = self.P_i / A_i
        
        self.q = zeros(n)
        self.q[0] = self.q_internodal[0] - (self.q_internodal[1]-self.q_internodal[0])/2
        self.q[-1] = self.q_internodal[-1]+(self.q_internodal[-1]-self.q_internodal[-2])/2
        self.q[1:-1] = (self.q_internodal[1:] + self.q_internodal[:-1])/2
