from iapws import IAPWS97
# ~ from CoolProp.CoolProp import PropsSI
from numpy import log10, zeros, linspace, ones

import SinglePhaseFlow

# ~ waterTable = "coolprop" # "coolprop"

def slipChisholm(rho_l, rho_g, x):
    """
    skluzový poměr
    ---
    D. Chisholm (1983), "Two-Phase Flow in Pipelines and Heat Exchangers"
    Longman Higher Education, 1983. ISBN 0-7114-5748-4
    ---
    rho_l - hustota kapalné fáze [kg/m3]
    rho_g - hustota plynné fáze [kg/m3]
    x - suchost [-]
    -->
    S - skluz v_g/v_l [-]
    """
    S = (1-x*(1-rho_l/rho_g))**.5
    return S

def velocityXflow(rho_l, rho_g, m, A, alpha, x, slip):
    """
    Výpočet rychlostí fází ve 2f směsi
    ---
    rho_l - hustota kapalné fáze [kg/m3]
    rho_g - hustota plynné fáze [kg/m3]
    m - celkový průtok [kg/s]
    A - průtočný průřez [m2]
    alpha - plošný podíl páry [-]
    x - statická suchost [-]
    slip - skluzový poměr [-]
    -->
    v_l - ryhlost kapalné fáze [m/s]
    v_g - ryhlost plynné fáze [m/s]
    j_l - obejomý tok kapalné fáze [m/s]
    j_g - obejomý tok plynné fáze [m/s]
    x_flow - průtočná suchost [-]
    """
    A_g = alpha*A
    A_l = (1-alpha)*A
    v_l = (1-x)*m / (A_l*rho_l)
    v_g = slip*v_l
    
    m_g = A_g*v_g*rho_g
    m_l = A_l*v_l*rho_l
    
    x_flow = m_g/(m_g+m_l)
    
    j_l = (1-alpha)*v_l
    j_g = alpha*v_g
    
    return v_l, v_g, j_l, j_g, x_flow, m_l, m_g

def desnsityMixture(rho_l,rho_v,x,slip):
    """
    hustota 2f směsi
    rho_l - hustota kapaliny [kg/m3]
    rho_v - hustota páry [kg/m3]
    x - suchost [-]
    slip - skluz [-]
    -->
    rho_m - hustota směsi [kg/m3]
    """
    alphaH, alphaDF = voidFracX(rho_l, rho_v, x, slip)
    rho_m = alphaDF*rho_v+(1-alphaDF)*rho_l
    return rho_m

def voidFracX(rho_l, rho_g, x, slip):
    """
    podíl páry
    rho_l - hustota kapaliny [kg/m3]
    rho_v - hustota páry [kg/m3]
    x - suchost [-]
    slip - skluz
    -->
    alphaH - podíl páry ve směsi HEM [-]
    alphaDF - podíl páry ve směsi Drift-Flux [-]
    """
    if x == 0:
        return(0,0)
    else:
        alphaDF = 1 / (1 + (1-x)/x * slip * rho_g/rho_l)
        alphaH = 1 / (1 + (1-x)/x * rho_g/rho_l)
        return(alphaH, alphaDF)

def bubbleRadius(rho_l, rho_g, sigma):
    """
    Poloměr parních bublin
    """
    dRho = rho_l-rho_g
    Rb = 0.5 * (sigma/(dRho*9.81))**.5
    return Rb

def eqnRohsenow(p,q):
    """
    p - tlak [Pa]
    q - tepelný tok [W/m2]
    -
    Rohsenov 1952, Heat Transfer - Cegel str. 522
    -
    q = liquid.mu*h_fg * (g*(liquid.rho-vapor.rho)/liquid.sigma)**.5 * ((liquid.cp*1e3*(Tw-T_sat)/(C_sf*h_fg*Pr**n))**3
    -
    Fluid-Heating Surface Combination Csf n
    Water–copper (polished) 0.0130 1.0
    Water–copper (scored) 0.0068 1.0
    Water–stainless steel (mechanically polished) 0.0130 1.0
    Water–stainless steel (ground and polished) 0.0060 1.0
    Water–stainless steel (teflon pitted) 0.0058 1.0
    Water–stainless steel (chemically etched) 0.0130 1.0
    Water–brass 0.0060 1.0
    Water–nickel 0.0060 1.0
    Water–platinum 0.0130 1.0
    n-Pentane–copper (polished) 0.0154 1.7
    n-Pentane–chromium 0.0150 1.7
    Benzene–chromium 0.1010 1.7
    Ethyl alcohol–chromium 0.0027 1.
    Carbon tetrachloride–copper 0.0130 1.7
    Isopropanol–copper 0.0025 1.7
    --->
    h - součinitel přestupu tepla [W/m2/K]
    dT - tepelný spád u stěny [K]
    """
    C_sf = .013
    n = 1.
    
    # ~ if waterTable == "iapws":
    vapor = IAPWS97(P=p/1e6,x=1)
    liquid = IAPWS97(P=p/1e6,x=0)
    Pr = liquid.Prandt
    h_fg = (vapor.h-liquid.h)*1e3
    mu = liquid.mu
    rho_l = liquid.rho
    rho_v = vapor.rho
    sigma = liquid.sigma
    cp_l = liquid.cp*1e3
        
    # ~ elif waterTable == "coolprop":
        # ~ rho_v = PropsSI('D', 'P', p, 'Q', 1, 'H2O')
        # ~ rho_l = PropsSI('D', 'P', p, 'Q', 0, 'H2O')
        # ~ Pr = PropsSI('Prandtl', 'P', p, 'Q', 0, 'H2O')
        # ~ h_fg = PropsSI('H', 'P', p, 'Q', 1, 'H2O') - PropsSI('H', 'P', p, 'Q', 0, 'H2O')
        # ~ mu = PropsSI('V', 'P', p, 'Q', 0, 'H2O')
        # ~ sigma = PropsSI('I', 'P', p, 'Q', 0, 'H2O')
        # ~ cp_l = PropsSI('C', 'P', p, 'Q', 0, 'H2O')
    
    A = mu * h_fg
    B = (9.81*(rho_l - rho_v) / sigma)**.5
    C = (cp_l/(C_sf*h_fg*Pr**n))**3
    
    M = q/(A*B*C)
    dT = M**0.33
    
    h = q/dT
    
    return h, dT

def eqnKruzilin(p,q):
    """
    Kružilinova rovnice pro přestup tepla
    ----
    p - tlak [MPa]
    q - tepelný tok [W/m2]
    Základy sdílení tepla Michejev 1952, str.131-133
    pozn. 0.2 - 100 atm
    --->
    h - součinitel přestupu tepla [W/m2/K]
    dT - tepelný spád u stěny [K]
    """
    
    # převod na kcal/m2/h
    q /= 1.1625
    
    # převod na atm
    p /= 1e6
    p *= 9.869
    
    # součinitel přestupu tepla kcal/m2/h/K
    h = 2.53 * p**.178 * q**.7
    
    # převod na W/m2/K
    h *= 1.1625
    
    dT = q/h
    
    return h, dT

def eqnSazima(p, q):
    """
    Součinitel přestupu tepla dle Sazimy
    ---
    p - tlak [Pa]
    q - tepelný tok [W/m2]
    --->
    h - součinitel přestupu tepla [W/m2/K]
    dT - tepelný spád u stěny [K]
    """
    # ~ if waterTable == "iapws":
    vapor = IAPWS97(P=p/1e6,x=1)
    liquid = IAPWS97(P=p/1e6,x=0)
    Pr = liquid.Prandt
    h_fg = (vapor.h-liquid.h)*1e3
    mu = liquid.mu
    rho_l = liquid.rho
    rho_v = vapor.rho
    sigma = liquid.sigma
    cp_l = liquid.cp*1e3
        
    # ~ elif waterTable == "coolprop":
        # ~ rho_v = PropsSI('D', 'P', p, 'Q', 1, 'H2O')
        # ~ rho_l = PropsSI('D', 'P', p, 'Q', 0, 'H2O')
        # ~ Pr = PropsSI('Prandtl', 'P', p, 'Q', 0, 'H2O')
        # ~ h_fg = PropsSI('H', 'P', p, 'Q', 1, 'H2O') - PropsSI('H', 'P', p, 'Q', 0, 'H2O')
        # ~ mu = PropsSI('V', 'P', p, 'Q', 0, 'H2O')
        # ~ sigma = PropsSI('I', 'P', p, 'Q', 0, 'H2O')
        # ~ cp_l = PropsSI('C', 'P', p, 'Q', 0, 'H2O')
    
    K = (sigma/(rho_l-rho_v) / 9.81)**.5
    h = 76.923 * (cp_l) / Pr**1.7 * (mu/K)**(1/3) * (q/h_fg)**(2/3)
    dT = q/h
    
    return h, dT

def eqnChen(G,Dh,x,p,h,q,dTw):
    """
    Chenova korelace pro přestup tepla
    ---
    G - hmotnostní tok [kg/m2/s]
    Dh - hydraulický průměr [m]
    x - suchost [-]
    p - tlak [Pa]
    h - entalpie [J/kg]
    q - tepelný tok [W/m2]
    dTw - tepelný spád u stěny [K]
    --->
    h_2f - součinitel přestupu tepla [W/m2/K]
    dTw_chen - tepelný spád u stěny [K]
    """
    # ~ if waterTable == "iapws":
    vapor = IAPWS97(P=p/1e6,x=1)
    liquid = IAPWS97(P=p/1e6,x=0)
    state = IAPWS97(P=p/1e6,h=h/1e3)
    Pr = liquid.Prandt
    h_fg = (vapor.h-liquid.h)*1e3
    mu = liquid.mu
    rho_l = liquid.rho
    rho_v = vapor.rho
    sigma = liquid.sigma
    sigma_s = state.sigma
    cp_l = liquid.cp*1e3
    T_l = liquid.T
    P_sat_wall = IAPWS97(T=T_l+dTw,x=0).P*1e6
    k = liquid.k
    
    # ~ elif waterTable == "coolprop":
        # ~ rho_v = PropsSI('D', 'P', p, 'Q', 1, 'H2O')
        # ~ rho_l = PropsSI('D', 'P', p, 'Q', 0, 'H2O')
        # ~ Pr = PropsSI('Prandtl', 'P', p, 'Q', 0, 'H2O')
        # ~ h_fg = PropsSI('H', 'P', p, 'Q', 1, 'H2O') - PropsSI('H', 'P', p, 'Q', 0, 'H2O')
        # ~ mu = PropsSI('V', 'P', p, 'Q', 0, 'H2O')
        # ~ sigma = PropsSI('I', 'P', p, 'Q', 0, 'H2O')
        # ~ sigma_s = PropsSI('I', 'P', p, 'H', h, 'H2O')
        # ~ cp_l = PropsSI('C', 'P', p, 'Q', 0, 'H2O')
        # ~ T_l = PropsSI('T', 'P', p, 'Q', 0, 'H2O')
        # ~ P_sat_wall = PropsSI('P', 'T', T_l+dTw, 'Q', 0, 'H2O')
        # ~ k = PropsSI('L', 'P', p, 'Q', 0, 'H2O')
    
    # ~ P_sat_wall = IAPWS97(T=liquid.T+dTw,x=0).P*1e6
    dP_sat = abs(P_sat_wall-p)
    
    # Reynolds - kapalná fáze
    Re_l = G*(1-x)*Dh/mu
    
    # Martinelliho parametr
    x_tt = ((1-x)/x)**.9 * (rho_v/rho_l)**.5 * (mu/rho_v)**.1
    
    if x_tt >= 5:
        F = 1
    else:
        F = 3*(1/x_tt)**.684
    
    # součinitel přestupu tepla pro nucenou konvekci
    h_FC = 0.023 * Re_l**.8 * Pr**.4 * (k/Dh) * F
    
    # Reynoldsovo číslo
    Re = Re_l*F**1.25
    
    # parametr S (podíl efektivního přehřátí a celkového přehřátí u stěny (dT_e/dT)**0.99 )
    if Re < 4e5:
        S = -0.5*log10(Re) + 2.9
    else:
        S = 0.1
    
    # součinitel přestupu tepla pro bublinkový var
    A = k**.79 * cp_l**.45 * rho_l**.49
    B = sigma_s**.5 * mu**.29 * h_fg**.24 * rho_v**.24
    h_NB = 1.22e-3 * A/B * dTw**.24 * dP_sat**.75 * S
    
    h_2f = h_NB + h_FC
    dTw_chen = q/h_2f
    
    return h_2f, dTw_chen

class ColdSideClass:
    def __init__(self, Ts, Tc, Power, A_I, A_II, m, mc, n_node):
        # základní parametry ---------------------------
        self.Ts = Ts
        self.Tc = Tc
        self.A_I = A_I
        self.A_II = A_II
        self.m = m
        self.mc = mc
        
        self.Ps = IAPWS97(T=Ts+273.15,x=0).P*1e6
        
        self.G_I = m/A_I
        self.G_II = m/A_II
        
        # teplota na vstupu k ohřívané stěně
        self.Tin = (Ts*(m-mc) + Tc*mc)/m
        
        # parametry vody na vstupu ------------------------
        self.water_in = IAPWS97(P=self.Ps/1e6,T=self.Tin+273.15)
        self.liquid = IAPWS97(P=self.Ps/1e6,x=0)
        self.vapor = IAPWS97(P=self.Ps/1e6,x=1)
        
        # entalpie
        self.dH = Power/self.m
        self.dH_i = self.dH/n_node
        H_in = self.water_in.h*1e3
        H_out = H_in + self.dH
        self.H = linspace(H_in, H_out, num=n_node+1)
    
    def solveTwoPhaseParameters(self, model):
        self.model = model
        
        # poloměr bublin
        self.Rb = bubbleRadius(self.liquid.rho, self.vapor.rho, self.water_in.sigma)
        
        n = len(self.H)
        # alokace polí -----------------------------------
        # teploty
        self.T = zeros(n)
        # hustoty
        self.Rho_l = zeros(n)
        self.Rho_g = zeros(n)
        self.Rho_m = zeros(n)
        self.Rho_hem = zeros(n)
        # skluz
        self.S = ones(n)
        # rychlosti
        self.v_l = zeros(n)
        self.v_g = zeros(n)
        self.j_l = zeros(n)
        self.j_g = zeros(n)
        # hmotnostní průtoky
        self.m_l = zeros(n)
        self.m_g = zeros(n)
        # suchosti
        self.x_st = zeros(n)
        self.x_flow = zeros(n)
        # plošný podíl páry
        self.alpha_hem = zeros(n)
        self.alpha_df = zeros(n)
        
        for i in range(n):
            
            # definice stavu 2f směsi
            mixture = IAPWS97(P=self.Ps/1e6,h=self.H[i]/1e3)
            
            # získání hustot z parních tabulek a uložení do příslušných polí
            self.Rho_l[i] = self.liquid.rho
            self.Rho_g[i] = self.vapor.rho
            self.Rho_hem[i] = mixture.rho
            
            # výpočet teploty směsi z parních tabulek
            self.T[i] = mixture.T-273.15
            
            # výpočet statické suchosti z parních tabulek
            self.x_st[i] = mixture.x
            
            # výpočet skluzu pokud je modelem Drift-Flux, pokud HEM tak skluz=1; v_l=v_g
            if 0 < self.x_st[i] < 1 and self.model == "Drift-Flux":
                self.S[i] = slipChisholm(self.Rho_l[i], self.Rho_g[i], self.x_st[i])
            else:
                pass
            
            # výpočet průtočné hustoty směsi
            if self.x_st[i] <= 0:
                self.Rho_m[i] = self.Rho_hem[i]
            else:
                self.Rho_m[i] = desnsityMixture(self.Rho_l[i], self.Rho_g[i], self.x_st[i], self.S[i])
            
            # výpočet plošného podílu páry
            self.alpha_hem[i], self.alpha_df[i] = voidFracX(self.Rho_l[i], self.Rho_g[i], self.x_st[i], self.S[i])
            
            # výpočet rychlostí, objemových a hmotnostních toků složek
            self.v_l[i], self.v_g[i], self.j_l[i], self.j_g[i], self.x_flow[i], self.m_l[i], self.m_g[i] =\
                velocityXflow(self.Rho_l[i], self.Rho_g[i], self.m, self.A_II, self.alpha_df[i], self.x_st[i], self.S[i])
        
        self.T_internodal = (self.T[0:-1] + self.T[1:])/2
    
    def solvePressureDrops(self, dZ, Dh1, Dh2, A2):
        n = len(self.H)-1
        
        self.dp_fric_in = zeros(n)
        self.dp_acc_in = zeros(n)
        self.dp_g_in = zeros(n)
        
        self.dp_fric_out = zeros(n)
        self.dp_acc_out = zeros(n)
        self.dp_g_out = zeros(n)
        
        rho_e_out = self.water_in.rho
        
        for i in range(n):
            
            # vzestupný proud --------------------------------
            rho_e = (self.Rho_m[i]+self.Rho_m[i+1])/2
            # třecí ztráta na elementu i
            self.dp_fric_in[i] = SinglePhaseFlow.dp_fric(rho_e, self.liquid, self.vapor, Dh1, self.G_I, self.x_flow[i], dZ[i])
            # akcelerační ztráta na elementu i
            self.dp_acc_in[i] = SinglePhaseFlow.dp_acc(self.Rho_m[i], self.Rho_m[i+1], self.G_I)
            # gravitační ztráta na elementu i
            self.dp_g_in[i] = SinglePhaseFlow.dp_grav(rho_e, dZ[i])
            
            # sestupný proud --------------------------------
            # třecí ztráta na elementu i
            self.dp_fric_out[i] = SinglePhaseFlow.dp_fric(rho_e_out, self.liquid, self.vapor, Dh2, self.G_II, 0, dZ[i])
            # akcelerační ztráta na elementu i
            # akcelerační ztráta = 0
            # gravitační ztráta (zisk) na elementu i
            self.dp_g_out[i] = SinglePhaseFlow.dp_grav(rho_e_out, dZ[i])
        
        self.dp_f = sum(self.dp_fric_in) + sum(self.dp_fric_out)
        self.dp_acc = sum(self.dp_acc_in)# + sum(self.dp_acc_out)
        self.dp_g = sum(self.dp_g_in) - sum(self.dp_g_out)
        
        self.dp_revers = 2 * 2 * 1.5 * ((self.G_II + self.G_I)/2/rho_e_out)**2 * .5 * rho_e_out
        
        self.dp_total = self.dp_f + self.dp_acc + self.dp_g + self.dp_revers
        
        if self.dp_total < 0:
            self.c_driven = abs(2*self.dp_total/rho_e_out)**.5
            self.m_driven = rho_e_out*self.c_driven*A2
        else:
            self.c_driven = 0
            self.m_driven = 0
        
    def solveHeatFlux(self, P_i, A_i):
        
        n = len(self.H)
        
        self.q_internodal = P_i / A_i
        
        self.q = zeros(n)
        self.q[0] = self.q_internodal[0] - (self.q_internodal[1]-self.q_internodal[0])/2
        self.q[-1] = self.q_internodal[-1]+(self.q_internodal[-1]-self.q_internodal[-2])/2
        self.q[1:-1] = (self.q_internodal[1:] + self.q_internodal[:-1])/2
    
    def solveHTC(self, Dh, h_eqn):
        n = len(self.H)
        
        self.h, self.dT_w = zeros(n), zeros(n)
        
        for i in range(n):
            if self.x_st[i] <= 0:
                Re = self.G_I * Dh / self.liquid.mu
                self.h[i], self.dT_w[i] = SinglePhaseFlow.heatTrasferRate(Re, self.liquid.Prandt, self.liquid.k, Dh, self.q[i])
            else:
                if h_eqn == "Rohsenow":
                    self.h[i], self.dT_w[i] =eqnRohsenow(self.Ps, self.q[i])
                elif h_eqn == "Kruzilin":
                    self.h[i], self.dT_w[i] = eqnKruzilin(self.Ps, self.q[i])
                elif h_eqn == "Chen":
                    # pro odhad teploty podchlazení u stěny rovnice Rohsenow
                    self.h[i], self.dT_w[i] = eqnRohsenow(self.Ps, self.q[i])
                    self.h[i], self.dT_w[i] = eqnChen(self.G_I, Dh, self.x_st[i], self.Ps, self.H[i], self.q[i], self.dT_w[i])
                elif h_eqn == "Sazima":
                    self.h[i], self.dT_w[i] = eqnSazima(self.Ps, self.q[i])
        self.h_internodal = (self.h[0:-1] + self.h[1:])/2
