import CoolProp.CoolProp as CP
import numpy as np

# hlavni parametry -> z reseni
T_sat = 154.04 # C
m_kond = 4.3e-3 # kg/s
P_sat = CP.PropsSI('P','T',T_sat+273.15,'Q', 0, 'H2O') # saturacni tlak

m_cool = 0.2 # kg/s
T_cool_in = 20 # C
k_tr = 15. # W/m.K

# geometrie
D = 0.016 # m
tl = 0.003
d_cool = 0.2

# hmotnistní podíly páry a plynu
w_g = 0.15 # plyn
w_w = 0.85 # voda

def Condensation(P_sat, T_sat, m_kond, T_cool_in, m_cool, d, tl, D, w_g, w_w, Power, k_tr):
    """
    P_sat - saturační tlak sekundární strany [Pa]
    T_sat - saturační teplota sekundární strany [C]
    m_kond - parní výkon sekundární strany [kg/s]
    d - průměr trubky chladícího okruhu [m]
    D - průměr terciálního okruhu [m]
    w_w - hmotnostní koncentrace páry [kg/kg]
    w_g - hmotnostní koncentrace Argonu [kg/kg]
    """
    
    n_node = 100
    
    # geometrie anuloidu
    L = np.pi*D
    dL = L/(n_node-1)
    d_in = d - 2*tl
    d_str = (d_in+d)/2
    S = 4 * np.pi * (D/2) * (d_str/2)
    dS = S/(n_node-1)
    A_in = np.pi * d_in**2 / 4
    
    # vlastnosti chladiva
    dH_cool = Power/m_cool
    H_cool_in = CP.PropsSI('H','T', T_cool_in+273.15,'P', 1e6, 'H2O')
    H_cool_out = H_cool_in + dH_cool
    T_cool_out = CP.PropsSI('T', 'P', 1e6, 'H', H_cool_out, 'H2O') - 273.15
    
    H_cool = np.linspace(H_cool_in, H_cool_out, num=n_node)
    T_cool = CP.PropsSI('T', 'P', 1e6, 'H', H_cool, 'H2O') - 273.15
    Rho_cool = CP.PropsSI('D', 'P', 1e6, 'H', H_cool, 'H2O')
    
    c_cool = m_cool / (A_in * Rho_cool)
    
    Pr_cool = CP.PropsSI('Prandtl', 'P', 1e6, 'H', H_cool, 'H2O')
    Re_cool = Rho_cool * c_cool * d_in / CP.PropsSI('V', 'P', 1e6, 'H', H_cool, 'H2O')
    Nu_cool = 0.023 * Re_cool**0.8 * Pr_cool**0.4
    h_cool = Nu_cool * CP.PropsSI('L', 'P', 1e6, 'H', H_cool, 'H2O') / d_in
    
    # molární hmotnosti
    M_g = 39.948 # g/mol
    M_w = 18.01528 # g/mol

    # molární podíly
    x_g = (w_g/M_g)/(w_g/M_g + w_w/M_w)
    x_w = (w_w/M_w)/(w_g/M_g + w_w/M_w)

    # parciální tlak
    P_g = P_sat*x_g
    P_w = P_sat*x_w

    T_wb = CP.PropsSI('T','P',P_w,'Q', 1, 'H2O')-273.15

    # podchlazení stěny
    dT_w = 40.   # C
    T_w = T_wb - dT_w

    # interface
    T_i = T_w + 0.25*dT_w

    P_wi = CP.PropsSI('P','T',T_i+273.15,'Q', 0, 'H2O')
    P_gi = P_sat - P_wi

    x_wi = P_wi/P_sat
    x_gi = P_gi/P_sat

    w_wi = (x_wi*M_w)/(x_wi*M_w + x_gi*M_g)
    w_gi = (x_gi*M_w)/(x_wi*M_w + x_gi*M_g)

    # inicializace smesi a smesovaci pravidlo pro CoolProp
    CP.apply_simple_mixing_rule('Argon', 'Water', 'linear')
    pair_and_fractions_b = 'Argon['+str(x_g)+']&Water['+str(x_w)+']'
    pair_and_fractions_i = 'Argon['+str(x_gi)+']&Water['+str(x_wi)+']'

    # vlastnosti smesi na interface a v prostoru + prumer
    Rho_mix_b = CP.PropsSI('Dmass','T',T_sat+273.15,'P',P_sat, pair_and_fractions_b)
    Rho_mix_i = CP.PropsSI('Dmass','T',T_i+273.15,'P',P_sat, pair_and_fractions_i)
    Rho_mix = (Rho_mix_b + Rho_mix_i)/2

    k_mix_b = CP.PropsSI('L','T',T_sat+273.15,'P',P_sat, pair_and_fractions_b)
    k_mix_i = CP.PropsSI('L','T',T_i+273.15,'P',P_sat, pair_and_fractions_i)
    k_mix = (k_mix_b + k_mix_i)/2

    c_mix_b = CP.PropsSI('C','T',T_sat+273.15,'P',P_sat, pair_and_fractions_b)
    c_mix_i = CP.PropsSI('C','T',T_i+273.15,'P',P_sat, pair_and_fractions_i)
    c_mix = (c_mix_b + c_mix_i)/2

    nu_mix_b = CP.PropsSI('V','T',T_sat+273.15,'P',P_sat, pair_and_fractions_b)
    nu_mix_i = CP.PropsSI('V','T',T_i+273.15,'P',P_sat, pair_and_fractions_i)
    nu_mix = (nu_mix_b + nu_mix_i)/2

    # podobnostní čísla
    beta = 2/(T_sat+T_i)
    Gr = beta * (T_sat-T_i) * 9.81 * d**3 / (nu_mix / Rho_mix)**2
    # ~ Gr = 9.81 * Rho_mix_b * d**3 * (Rho_mix_i - Rho_mix_b) / nu_mix**2
    Pr = (nu_mix * c_mix) / k_mix
    Ra = Gr*Pr
    Nu_D = (0.6 + 0.387*Ra**(1/6)/(1+(0.559/Pr)**(9/16))**(8/27))**2

    # součinitel přestupu tepla konvekce trubky
    h_D = Nu_D*k_mix/d

    # filmová složka součinitele přestupu tepla
    T_cond = (T_w + T_i)/2

    Rho_cond_g = CP.PropsSI('D','T',T_i+273.15,'Q', 1, 'H2O')
    h_cond_g = CP.PropsSI('H','T',T_i+273.15,'Q', 1, 'H2O')

    Rho_cond_l = CP.PropsSI('D','T',T_i+273.15,'Q', 0, 'H2O')
    h_cond_l = CP.PropsSI('H','T',T_i+273.15,'Q', 0, 'H2O')
    k_cond_l = CP.PropsSI('L','T',T_i+273.15,'Q', 0, 'H2O')
    nu_cond_l = CP.PropsSI('V','T',T_i+273.15,'Q', 0, 'H2O')
    c_cond_l = CP.PropsSI('C','T',T_i+273.15,'Q', 0, 'H2O')

    l_cond = h_cond_g - h_cond_l

    h_F = 0.725 * (9.81 * Rho_cond_l * (Rho_cond_l-Rho_cond_g) * k_cond_l**3 * l_cond / (d * nu_cond_l * (T_i-T_w) ))**0.25
    
    h = h_D + h_F
    
    # prostup tepla válcovou stěnou
    b = 2*np.pi*k_tr*(T_i-T_cool)
    a1 = k_tr/(d_in/2)/h_cool
    a2 = k_tr/(d/2)/h
    c = np.log((d/2)/(d_in/2))
    Q = b / (a1 + a2 + c) * dL
    Q[0] /= 2
    Q[-1] /= 2
    P_transf = sum(Q)
    
    print(Power, P_transf)
    
    return T_cond, T_cool_out, h

print(Condensation(P_sat, T_sat, m_kond, T_cool_in, m_cool, D, tl, d_cool, w_g, w_w, 6500, k_tr))
